PROGRAM main

  ! main program of the builder tool
  
  USE odessa
  
  ! variable initialized by the main program  
  USE builder, ONLY : builddir,errmes,nerror,analbase,echo,print_manual
  
  ! the only subroutine
  USE builder, ONLY : build_init,build_task
  
  IMPLICIT NONE
  
  INTEGER        :: kunit      ! file unit for reading or saving the build status
  INTEGER        :: status     ! task status
  TYPE(odbase)   :: config     ! configuration base
  TYPE(odbase)   :: rule       ! rule base
  TYPE(odbase)   :: build      ! previous build state
  TYPE(odbase)   :: menu       ! the menu when using the -gui option
  TYPE(odbase)   :: menubase   ! an empty base
  TYPE(odbase)   :: menuresu   ! the result of the GUI
  CHARACTER(32)  :: cplatform  ! retained platform
  CHARACTER(255) :: fullname   ! name of the saving file
  CHARACTER(512) :: command    ! command
  CHARACTER(255) :: dirname    ! directory containing the executable program of the builder
  CHARACTER(255) :: dirdat     ! directory of data files used by the builder
  CHARACTER(255) :: basename   ! = "builder"
  CHARACTER(8)   :: suffix     ! either ' ' or '.exe'
  CHARACTER(255) :: carg       ! command line argument
  CHARACTER(255) :: cconf      ! name of the configuration file
  CHARACTER(255) :: carch      ! name of the computer architecture
  INTEGER        :: i,k        ! loop index
  
  CALL odinit ! to initialize ODESSA
  
  ! builder root directory
  
  CALL get_command(command)
  i=index(command,'builder')
  IF(i <= 0) THEN
    WRITE(*,*) 'impossible to analyze the command'
    STOP
  ENDIF
  i=i+7
  k=index(command(i:),' ')
  i=i+k-1
  CALL odfile_splitname(command(1:i-1),dirname,basename,suffix)
  fullname=dirname
  CALL odfile_splitname(fullname,dirname,basename,suffix)
  fullname=dirname
  CALL odfile_splitname(fullname,dirname,basename,suffix)
  
  ! directory containing private data files
  
  CALL odfile_name(dirdat,dirname,'dat')
  
  ! arguments
  
  cconf="builder.cfg"
  cplatform=' '
  
  i=0
  DO
    i=i+1
    IF(i > command_argument_count()) EXIT
    CALL get_command_argument(i,carg)
    IF(carg == '-f') THEN
      i=i+1
      CALL get_command_argument(i,cconf)
    ELSE IF(carg == '-h') THEN ! short user manual
      CALL print_manual
      STOP
    ELSE IF(carg == '-v') THEN ! verbose mode
      echo=6
    ELSE IF(carg == '-d') THEN
      i=i+1
      CALL get_command_argument(i,dirname)
      status=odfile_chdir(dirname)
      IF(status /= 0) THEN
        WRITE(*,*) 'impossible to go to the directory ',TRIM(dirname)
        STOP 1
      ENDIF
    ELSE IF(carg == '-gui') THEN
      CALL odfile_name(fullname,dirdat,'builder.men')
      CALL odread(menu,fullname,echo=0)
      CALL odmenuedit(menu,menubase,menuresu)
      CALL odprint(menuresu)
      CALL odget(menuresu,'status',status)
      IF(status /= 0) STOP "giving up the code builder"
      CALL odget(menuresu,'director',fullname)
      i= odfile_chdir(fullname) ! changing the current directory
      IF(i /= 0) STOP "giving up the code builder"
      CALL odget(menuresu,'config',cconf)
      CALL odget(menuresu,'platform',cplatform)
      EXIT
    ELSE IF(carg == '-clean') THEN
      i=i+1
      CALL get_command_argument(i,cconf)
      IF(cconf /= ' ') &
      status = odfile_rmdir("bin/"//cconf)
    ELSE
      cplatform=carg
    ENDIF
  ENDDO
  
  ! Locating the configuration file
  
  IF(.NOT.odfile_exist(cconf)) THEN
    ! looking for the file from the builder root directory
    CALL odfile_name(fullname,dirname,cconf)
    write(*,*) 'testing ',TRIM(fullname)
    IF(.NOT.odfile_exist(fullname)) THEN
      CALL odfile_name(fullname,dirdat,cconf)
      write(*,*) 'testing ',TRIM(fullname)
      IF(.NOT.odfile_exist(fullname)) THEN
        WRITE(*,*) 'impossible to find the file ',TRIM(cconf)
        STOP
      ELSE
        cconf=fullname
      ENDIF
    ELSE
      cconf=fullname
    ENDIF
  ENDIF
  
  ! initializing the configuration base
  
  CALL odinit(config)   ! configuration base containing the project definition
  CALL odinit(analbase) ! the analyzer database use every where
  
  CALL odput(analbase,'argument',TRIM(cplatform))
  
  CALL odarch(carch)
  CALL odput(analbase,'computer',carch)
  
  ! loading the configuration file
  
  CALL odinit(config) ! configuration base
  CALL odread(config,cconf,analbase=analbase) ! reading the configuration file
  CALL odfile_name(fullname,dirdat,"builder.rul")
  CALL odinit(rule)
  CALL odread(rule,fullname,echo=0) ! reading the configuration file
  CALL odcheck(config,rule)  ! checking phase
  
  ! getting main options
  
  CALL build_init(config,cplatform)

  ! loading the previous build state
  
  CALL odfile_name(fullname,builddir,'.build')
  open(unit=99,file=fullname,form='UNFORMATTED',status='OLD',iostat=status)
  IF(status /= 0) THEN
    IF(.NOT.odfile_exist(builddir)) THEN
      status=odfile_mkdir(builddir)
      IF(status /= 0) STOP "impossible to create the build directory"
    ENDIF
    CALL odinit(build)
    kunit=0
  ELSE
    kunit=99
    CALL odload(build,kunit)
  ENDIF
  
  ! calling the recursive routine of the builder executing tasks in the right order
  
  CALL build_task(config,build,status,1)
  
  ! saving the current build state
  
  IF(kunit > 0) THEN
    REWIND(kunit)
  ELSE
    open(unit=kunit,file=fullname,form='UNFORMATTED',status='NEW',iostat=status)
    IF(status /= 0) THEN
      STOP "impossible to save the current build file"
    ENDIF
  ENDIF
  CALL odsave(config,kunit)
  CLOSE(kunit)
  IF(status == 0) THEN
    WRITE(*,'(1X,A)') "up-to-date project"
  ELSE
    WRITE(*,'(/,1X,A,/)') "Summary or errors when building the project :"
    DO i=1,nerror
      WRITE(*,'(3X,A)') TRIM(errmes(i))
    ENDDO
  ENDIF

END PROGRAM
